package de.fitko.fitconnect.core;

import com.google.inject.Inject;
import com.nimbusds.jose.jwk.RSAKey;
import de.fitko.fitconnect.api.domain.auth.OAuthToken;
import de.fitko.fitconnect.api.domain.model.metadata.Metadata;
import de.fitko.fitconnect.api.domain.model.metadata.attachment.Attachment;
import de.fitko.fitconnect.api.domain.model.metadata.data.Data;
import de.fitko.fitconnect.api.domain.model.submission.Submission;
import de.fitko.fitconnect.api.domain.model.submission.SubmissionForPickup;
import de.fitko.fitconnect.api.domain.validation.ValidationResult;
import de.fitko.fitconnect.api.services.Subscriber;
import de.fitko.fitconnect.api.services.auth.OAuthService;
import de.fitko.fitconnect.api.services.crypto.CryptoService;
import de.fitko.fitconnect.api.services.validation.MetadataValidator;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.List;
import java.util.UUID;

public class SubmissionSubscriber implements Subscriber {

    private static final Logger logger = LoggerFactory.getLogger(SubmissionSubscriber.class);

    private final OAuthService authService;
    private final CryptoService cryptoService;
    private final MetadataValidator metadataValidator;

    @Inject
    public SubmissionSubscriber(final OAuthService authService,
                                final CryptoService cryptoService,
                                final MetadataValidator metadataValidator) {
        this.authService = authService;
        this.cryptoService = cryptoService;
        this.metadataValidator = metadataValidator;
    }

    @Override
    public OAuthToken retrieveOAuthToken(final String clientId, final String clientSecret, final String... scope) {
        final OAuthToken token = authService.authenticate(clientId, clientSecret, scope);
        logger.debug("Successfully retrieved OAuth token: {}", token.getAccessToken());
        return token;
    }

    @Override
    public byte[] decryptStringContent(final RSAKey privateKey, final String encryptedContent) {
        return cryptoService.decryptBytes(privateKey, encryptedContent);
    }

    @Override
    public List<SubmissionForPickup> pollAvailableSubmissions(final UUID destinationId, final int limit, final int offset) {
        throw new UnsupportedOperationException("not yet implemented");
    }

    @Override
    public Submission getSubmission(final UUID submissionId) {
        throw new UnsupportedOperationException("not yet implemented");
    }

    @Override
    public List<Attachment> fetchAttachments(final UUID submissionId, final List<UUID> announcedAttachments) {
        throw new UnsupportedOperationException("not yet implemented");
    }

    @Override
    public ValidationResult validateEventLog(final UUID caseId) {
        throw new UnsupportedOperationException("not yet implemented");
    }

    @Override
    public ValidationResult validateMetadata(final Metadata metadata, final String schema) {
        throw new UnsupportedOperationException("not yet implemented");
    }

    @Override
    public ValidationResult validateData(final Data data) {
        throw new UnsupportedOperationException("not yet implemented");
    }

    @Override
    public ValidationResult validateAttachments(final List<Attachment> attachments) {
        throw new UnsupportedOperationException("not yet implemented");
    }

    @Override
    public void confirmValidSubmission() {
        throw new UnsupportedOperationException("not yet implemented");
    }

}
